/* #arfind.c 10.7.2013/SM (24.4.2014) (11.4.2017)
*/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <malloc.h>
#include <math.h>
#include <survo.h>
#include <survoext.h>
#include "ext_mat.h"

extern char *argv1;
static int i,n,m,k,j,q,i1;
static double *D,*H;
static double *X;
static double *C;

static char name[64];
static char *lab;

// MAT #ARFIND(n,matrix)

extern double pi;
extern double sgn();
extern double cot();

op__arfind()
        {
        char expr1[2*LLENGTH];
        double a,b,gg,gap;
        int odd=0;
        int c_q,c_factor,c_index,c_amod;
        int f;

        i=external_mat_init(1); if (i<0) return(1);
        if (g<4)
            {
            init_remarks();
            rem_pr("MAT #ARFIND(n,A)                                     ");
            rem_pr("http://www.survo.fi/papers/Roots2013.pdf             ");
            rem_pr("Numerical roots directly as cot expressions! (2017)  ");
            rem_pr("                                                     ");
            rem_pr("sign, q, factor, index, amod  values saved as A.MAT  ");
            rem_pr("Matrix of coefficients saved as Cn.MAT (2017)        ");
            wait_remarks(2);
            return(1);
            }

        n=atoi(word[2]);
        odd=1; if (n%2==0) odd=0;
        m=n/2;
        mat_alloc(&X,m,1); // Numerical roots
        for (i=0; i<m; ++i) X[i]=cot((2*i+1)*3.141592653589793/(2*n));
        // X corresponds to R/n.

        odd=n-2*m; k=2-odd;

        c_q=m;
        c_factor=2*m;
        c_index=3*m;
        c_amod=4*m;

        i=mat_alloc_lab(&T,m,5,&rlabT,&clabT);
        strcpy(clabT,"sign    q       factor  index   amod");
        numlab(rlabT,m,8);
        for (i=0; i<5*m; ++i) T[i]=0.0;
        D=malloc(m*sizeof(double));
        H=malloc(m*sizeof(double));

        for (i=1; i<=m; ++i)
            {
            D[i-1]=2*sin((double)(m+1-i)*pi/(double)n);
            T[c_amod+i-1]=(double)amod(n,2*i-1); // 24.4.2014
            }
        if (odd==0) D[0]=1;
        // D corresponds to E/n.

        i=1; q=1;
        while (i<=m)
            {
            for (j=1; j<=m; ++j)
                {
                a=cos(q*pi*(double)(2*j-k)/(double)(2*i-1));
                if (a<0.0) a=-1; else a=1;
                H[j-1]=a;
                }
           gg=0.0;
           for (j=1; j<=m; ++j) gg+=H[j-1]*D[j-1];
           if (gg<0.0) b=-1; else b=1; // sign

           if (i==1)
                {
                T[i-1]=b;
                T[c_q+i-1]=1;
                ++i; iprint(i); continue;
                }
           if (i>1)
                {
                if (T[c_factor+i-1]>0) { ++i; q=1; continue; }
                if (fabs(fabs(gg)-X[i-1])>0.00000001)
                    {
                    ++q;
                    if (q<i) continue;
                    else
                        {
                        // If no valid q is found,
                        // X[i-1] must be a 'factor' root.
                        a=X[i-1];
                        f=(int)(pi*(3*a+sqrt(12+9*a*a))/12+0.5);
                        if (n%f!=0)
                            {
                            // This error message should never appear!
                            sprintf(sbuf,"\n%d is not a factor of %d!",
                                          f,n);
                            sur_print(sbuf); getch();
                            return(1);
                            }
                        gap=n/f; // Recording other roots related to f
                        i1=i; q=1;
                        while (i1<=m)
                            {
                            T[c_factor+i1-1]=f;
                            T[c_index+i1-1]=q;
                            i1+=(int)gap; ++q;
                            }
                        ++i; iprint(i); q=1; continue;
                        }
                    }
                else
                    {
                    T[i-1]=b;
                    T[c_q+i-1]=q;
                    ++i; iprint(i); q=1;
                    continue;
                    }
                }
            } // i

        mT=m;
        nT=5;
        strcpy(exprT,"Exact_roots");
        i=save_T(word[3]);

        // 3.4.2017 Coefficient matrix -> Cn.MAT
        C=malloc(m*m*sizeof(double));
        for (i=0; i<m*m; ++i) C[i]=0.0;
        for (i=0; i<m; ++i)
            {
          for (j=0; j<m; ++j)
            C[i+m*j]=T[i]*sgn(cos(pi*T[i+m]*(2*j+odd)/(double)(2*i+1)));
            }
        sprintf(sbuf,"Coefficient_matrix_C%d_for_n=%d",n,n);
        sprintf(name,"C%d",n);

        lab=malloc(8*m);
        numlab(lab,m,8);
        matrix_save(name,C,m,m,lab,lab,8,8,-1,sbuf,0,0);

        external_mat_end(argv1);
        return(1);
        }

iprint(i)
int i;
    {
    sprintf(sbuf," %d",i); sur_print(sbuf); // 13.4.2017
    return(1);
    }

amod(n,i)
int n,i;
    {
    int mod;
// amod(N,X):=if(mod(N,X)>int(X/2))then(X-mod(N,X))else(mod(N,X))
    mod=n%i;
    if (mod>(int)(i/2)) mod=i-mod;
    return(mod);
    }

double sgn(x)
double x;
    {
    if (x>0.0) return(1.0);
    if (x<0.0) return(-1.0);
    return (0.0);
    }

double cot(double x)
{ return (1/tan(x)); }

